@@ -12,10 +12,12 @@ from __future__ import absolute_import
 import time
 import logging
 
+from django import VERSION as DJANGO_VERSION
 from django.conf import settings
 from django.core.exceptions import SuspiciousOperation
 from django.http import HttpRequest
 from django.template import TemplateSyntaxError
+from django.utils.datastructures import MultiValueDict
 
 try:
     # support Django 1.9
@@ -26,21 +28,22 @@ except ImportError:
 
 from raven.base import Client
 from raven.contrib.django.utils import get_data_from_template, get_host
-from raven.contrib.django.middleware import SentryLogMiddleware
-from raven.utils.wsgi import get_headers, get_environ
+from raven.contrib.django.middleware import SentryMiddleware
+from raven.utils.compat import string_types, binary_type, iterlists
+from raven.contrib.django.resolver import RouteResolver
+from raven.utils.wsgi import get_headers, get_environ, get_client_ip
 from raven.utils import once
 from raven import breadcrumbs
-from raven._compat import string_types, binary_type
 
 __all__ = ('DjangoClient',)
 
-VALID_HTTP_REQUESTS = (HttpRequest, )
 
-try:
-    from rest_framework.request import Request
-    VALID_HTTP_REQUESTS += (Request, )
-except ImportError:
-    pass
+if DJANGO_VERSION < (1, 10):
+    def is_authenticated(request_user):
+        return request_user.is_authenticated()
+else:
+    def is_authenticated(request_user):
+        return request_user.is_authenticated
 
 
 class _FormatConverter(object):
@@ -137,6 +140,7 @@ def install_sql_hook():
 
 class DjangoClient(Client):
     logger = logging.getLogger('sentry.errors.client.django')
+    resolver = RouteResolver()
 
     def __init__(self, *args, **kwargs):
         install_sql_hook = kwargs.pop('install_sql_hook', True)
@@ -147,19 +151,19 @@ class DjangoClient(Client):
     def install_sql_hook(self):
         install_sql_hook()
 
-    def get_user_info(self, user):
+    def get_user_info(self, request):
+
+        user_info = {
+            'ip_address': get_client_ip(request.META),
+        }
+        user = getattr(request, 'user', None)
+        if user is None:
+            return user_info
+
         try:
-            if hasattr(user, 'is_authenticated'):
-                # is_authenticated was a method in Django < 1.10
-                if callable(user.is_authenticated):
-                    authenticated = user.is_authenticated()
-                else:
-                    authenticated = user.is_authenticated
-                if not authenticated:
-                    return None
-
-            user_info = {}
-
+            authenticated = is_authenticated(user)
+            if not authenticated:
+                return user_info
             user_info['id'] = user.pk
 
             if hasattr(user, 'email'):
@@ -169,22 +173,18 @@ class DjangoClient(Client):
                 user_info['username'] = user.get_username()
             elif hasattr(user, 'username'):
                 user_info['username'] = user.username
-
-            return user_info
         except Exception:
             # We expect that user objects can be somewhat broken at times
             # and try to just handle as much as possible and ignore errors
             # as good as possible here.
-            return None
+            pass
+
+        return user_info
 
     def get_data_from_request(self, request):
         result = {}
 
-        user = getattr(request, 'user', None)
-        if user is not None:
-            user_info = self.get_user_info(user)
-            if user_info:
-                result['user'] = user_info
+        result['user'] = self.get_user_info(request)
 
         try:
             uri = request.build_absolute_uri()
@@ -210,6 +210,11 @@ class DjangoClient(Client):
                         data = request.POST or '<unavailable>'
                     except Exception:
                         data = '<unavailable>'
+                    else:
+                        if isinstance(data, MultiValueDict):
+                            data = dict(
+                                (k, v[0] if len(v) == 1 else v)
+                                for k, v in iterlists(data))
         else:
             data = None
 
@@ -263,9 +268,9 @@ class DjangoClient(Client):
             data = kwargs['data']
 
         if request is None:
-            request = getattr(SentryLogMiddleware.thread, 'request', None)
+            request = getattr(SentryMiddleware.thread, 'request', None)
 
-        is_http_request = isinstance(request, VALID_HTTP_REQUESTS)
+        is_http_request = isinstance(request, HttpRequest)
         if is_http_request:
             data.update(self.get_data_from_request(request))
 
@@ -299,3 +304,6 @@ class DjangoClient(Client):
             }
 
         return result
+
+    def get_transaction_from_request(self, request):
+        return self.resolver.resolve(request.path)
